"""A template Python module"""

__version__ = "6.0.0"
from .api import *  # noqa
from .calculate_eradication_progress import *  # noqa
from .calculate_intersect import *  # noqa
from .data_requirements_plot import *  # noqa
from .fit_ramsey_time_series import *  # noqa
from .plot_cpue_series import *  # noqa
from .plot_progress_probability import *  # noqa
from .resample_aerial_monitoring import *  # noqa
from .resample_raw_data import *  # noqa
from .set_data import *  # noqa
